def dumb_cartesian_product(L,*lists):
    """
    A cartesian product to use if itertools.product is not
    available on your platform (python < 2.6)
    """
    if not lists:
        for x in L:
            yield [x]
    else:
        for x in L:
            for y in cartesian_product(lists[0],*lists[1:]):
                y.append(x)
                yield y

try:
    from itertools import product as cartesian_product
except ImportError:
    cartesian_product = dumb_cartesian_product

def k_subsets_i(n, k):
    """
    Yield each subset of size k from the set of intergers 0 .. n - 1
    n -- an integer > 0
    k -- an integer > 0
    """
    # Validate args
    if n < 0:
        raise ValueError('n must be > 0, got n=%d' % n)
    if k < 0:
        raise ValueError('k must be > 0, got k=%d' % k)
    # check base cases
    if k == 0 or n < k:
        yield set()
    elif n == k:
        yield set(range(n))

    else:
        # Use recursive formula based on binomial coeffecients:
        # choose(n, k) = choose(n - 1, k - 1) + choose(n - 1, k)
        for s in k_subsets_i(n - 1, k - 1):
            s.add(n - 1)
            yield s
        for s in k_subsets_i(n - 1, k):
            yield s


def k_subsets(s, k):
    """
    Yield all subsets of size k from set (or list) s
    s -- a set or list (any iterable will suffice)
    k -- an integer > 0
    """
    s = list(s)
    n = len(s)
    for k_set in k_subsets_i(n, k):
        yield frozenset(s[i] for i in k_set)
