"""
Encode, Extract association rules, and Decode

"""

import os
import shutil

import generalized_multi
import encode
import ar
import io

def _main(info_logger, temporary_dir, dataset, minconf, output_file, sort_by="lift", ascending=False, opportunistic=False):
    
    dct_prefix = os.path.join(temporary_dir, "dictionary")

    # Encode itemsets
    dataset_encoded = os.path.join(temporary_dir, "%s_encoded" % (dataset, ))
    info_logger.info("Encoding itemsets: file %s on %s" % (dataset, dataset_encoded) )

    encode.main(dataset, dct_prefix, dataset_encoded, False, file_type=2, load_old_data=False)

    
    # Extracting AR
    temp_filename = os.path.join(temporary_dir, 'ar_encoded')

    info_logger.info("Launching ar.py. minconf = %s. Encoded association rules written on %s" % (minconf, temp_filename) )
    f = file(temp_filename, 'w')
    
    ar.main(dataset_encoded, minconf, info_logger, f, sort_by = sort_by, ascending = ascending, opportunistic=opportunistic)

    f.close()

    # Decoding association rules
    info_logger.info("Decoding itemsets" )
    ar_decoded = os.path.join(temporary_dir, "ar_decoded")
    encode.main(temp_filename, dct_prefix, ar_decoded, True, file_type=3, load_old_data=True)


    # Printing output
    if output_file is None:
        info_logger.info("Printing output to standard output" )
        for line in file(ar_decoded):
            print line,
    else:
        info_logger.info("Printing output to %s" % (output_file ,) )
        shutil.copyfile(ar_decoded, output_file)

def main(dataset, minconf, log_file, output_file, **kwargs):
    return io.wrap_passing_info_logger_and_tempdir(log_file, _main, dataset, minconf,  output_file, **kwargs)
