#!/usr/bin/python

try:
    import psyco
    psyco.full()
except:
    print "psyco disabled"

import sys
from optparse import OptionParser
from generalized_multi import glue_itemsets as app
import os

usage = \
"""
%prog [options] dataset minsup

Encode the dataset and extract itemsets

%prog -h for help
"""

available_generation_strategies = ["well-formed", "traditional", "genio", "traditional_all_generalizations"]
available_extraction_strategies = ["opportunistic", "traditional", "traditional_basic_pruning",\
                                  "opportunistic_basic_pruning", "sac", "opportunistic_post_pruning"]
available_print_excluded_strategies = ["extended", "compact", "ignore"]

def shell_ui(parser, options, args):    
    # No signals, no threads
    sys.setcheckinterval(10000)

    if len(args) < 2:
        parser.error("Missing argument")
        sys.exit()

    if options.candidate_generation_strategy.lower() not in available_generation_strategies:
        parser.error("candidate generation strategy can be %s" % ('|'.join(available_generation_strategies), ))
    if options.extraction_strategy.lower() not in available_extraction_strategies:
        parser.error("algorithm can be %s" % ('|'.join(available_extraction_strategies), ))
    if options.print_excluded_strategy.lower() not in available_print_excluded_strategies:
        parser.error("candidate generation strategy can be %s" % ('|'.join(available_print_excluded_strategies), ))
    try:
        dataset = args[0]
        minsup = int(args[1])
    except ValueError:
        parser.error("minsup must be an integer")

    log_file = options.log_file
    rules_file = options.rules_file
    output_file = options.output

    app.main(dataset, minsup, log_file, rules_file, output_file, 
             extraction_strategy = options.extraction_strategy.lower(),
             candidate_generation_strategy = options.candidate_generation_strategy.lower(),
             optimize = options.optimize,
             ensure_subsets = options.ensure_subsets, 
             print_excluded_strategy = options.print_excluded_strategy)

if __name__ == '__main__':
    # Process arguments
    parser = OptionParser(usage=usage)
    parser.add_option("-g", "--gui", dest="gui",default=False,action="store_true",
              help="Run a minimal GUI.")

    parser.add_option("-r", "--rules", dest="rules_file",default=None,
              help="read generalization rules from FILE. Default: None", metavar="FILE")

    parser.add_option("-m", "--mark-subsets", dest="ensure_subsets",default=False,action="store_true",
                      help="If not opportunistic techniques are used,"+\
                          "and generalizations is involved, ensure the inclusion" +\
                          " of subitemsets of frequent itemsets")

    parser.add_option("-p", "--print-excluded-strategy", dest="print_excluded_strategy",default="extended",
                      help="If opportunistic techniques are used,"+\
                          "and generalizations is involved, enrich each itemset with the set of excluded itemsets."+\
                          "Default: 'extended'",
                      metavar='|'.join(available_print_excluded_strategies))

    parser.add_option("-l", "--log", dest="log_file",default=os.devnull,
              help="log on LOGFILE. Default: %s" % (os.devnull,) , metavar="LOGFILE")
    
    parser.add_option("-c", "--candidate-generation-strategy",
                      dest="candidate_generation_strategy",type="string", default="genio",
                      help="Choose the candidate generation strategy to use. Default: 'genio'",
                      metavar='|'.join(available_generation_strategies))
    
    parser.add_option("-o", "--output", dest="output",default=None,
              help="write itemset to OUTFILE. Default: standard output" , metavar="OUTFILE")
    
    parser.add_option("-x", "--extraction-strategy", dest="extraction_strategy",default="opportunistic",
                      help="Choose the extraction generation strategy to use. Default: 'opportunistic'" ,
                      metavar='|'.join(available_extraction_strategies))
    
    parser.add_option("-O", "--optimize", dest="optimize",action="count",default=0,
                  help="optimize:\n\t-O disable garbage collector (insane).. ")
    
    (options, args) = parser.parse_args()

    if options.gui:
        from generalized_multi import tkgui
        tkgui.fim_gui(parser, options, args, available_generation_strategies,
                  available_extraction_strategies, available_print_excluded_strategies)
    else:
        shell_ui(parser, options, args)
