import Tkinter as T
import threading
import Queue
import sys
import itertools
import copy
from tkMessageBox import showerror

widget_global_options = {'sticky' : T.W+T.E+T.N+T.S}

def build_root():
    root = T.Tk()
    root.title(sys.argv[0])
    root.grid_columnconfigure(1, weight=1)
    return root

def ensure_string_is_not_empty(val):
    st = str(val)
    assert(len(st) > 0)
    return st

def choose_file(entry_to_set, filedialog):
    def _choose_file():
        entry_to_set.delete(0, T.END)
        entry_to_set.insert(0, filedialog())
    return _choose_file


def get_options_reader(dct):
    def _options_reader(widget, key, converter, error_string):
        try:
            dct[key] = converter(widget.get())
        except:
            dct['__ex'] += "\n%s"  % (error_string,)
    return _options_reader


def set_status_updater(root,status, job_runner, interval=2):
    def update_status():
        enqueued = job_runner.enqueued_jobs()
        status.config(text= "%s processes enqueued" % (enqueued,))
        root.after(int(1000*interval), update_status)
    update_status()

class JobRunner(object):
    """
    A single thread for doing the work!
    The generalized_multi code is not thread safe!
    """
        
    def __init__(self, spawn = False, func=None):
        self.queue = Queue.Queue()
        if spawn == True:
            self.t = threading.Thread(target=func(self.queue))
            self.t.start()

    def enqueued_jobs(self):
        return self.queue.qsize()

    def enqueue_job(self, cmd):
        _cmd = copy.deepcopy(cmd)
        self.queue.put(_cmd)

    def stop(self):
        self.queue.put({'__stop' : True})

    def wait(self):
        self.t.join()


def get_worker(queue, func):
    def _worker():
        while True:
            cmd = queue.get()
            if '__stop' in cmd:
                print "Jobs completed!"
                break
            func(cmd)

    return _worker


class GUI(object):
    def __init__(self, job_runner):
        self.job_runner = job_runner
        self.free_row = itertools.count()

        self.root = build_root() 
        status = T.Label(self.root, text="0 processes enqueued")
        status.grid(row=self.free_row.next(), column=0, columnspan=3)
        set_status_updater(self.root, status, self.job_runner)

        self.root.protocol("WM_DELETE_WINDOW", self.cmd_quit_when_completed)
        self.widget_infos = []

    def cmd_quit_when_completed(self):
        print "Close UI... ", 
        self.root.destroy()
        print "Wait for jobs completion"
        self.job_runner.stop()

    def next_row(self):
        return self.free_row.next()
        
    def cmd_enqueue_job(self):
        options =  {'__ex': ""}
        options_reader = get_options_reader(options)
        map(lambda x: options_reader(*x), self.widget_infos)
        if options['__ex'] != "":
            showerror("Error!", options['__ex'])
            options['__ex'] = ""
            return
        self.job_runner.enqueue_job(options)

    def run(self):
        r = self.next_row()
        T.Button(self.root, text="Enqueue", 
                 command = self.cmd_enqueue_job).grid(row=r, column=0, columnspan=2, **widget_global_options)
        T.Button(self.root, text="Quit the GUI", 
                 command = self.cmd_quit_when_completed).grid(row=r, column=2, **widget_global_options)
        self.root.mainloop()
       
    def file_chooser_row(self, text, filedialog, key, validator, descrition_label):
        root = self.root
        row = self.next_row()
        T.Label(root, text=text).grid(row=row, column = 0, **widget_global_options)
        e = T.Entry(root)
        e.grid(row=row, column=1, **widget_global_options)
        T.Button(root, text="Browse...", \
                     command=choose_file(e, filedialog)).grid(row=row, column=2, **widget_global_options)
        self.widget_infos.append((e, key, validator, descrition_label))
        return e

    def entry_row(self, text, key, validator, descrition_label):
        root = self.root
        row = self.next_row()
        T.Label(root, text=text).grid(row=row, column = 0, **widget_global_options)
        e = T.Entry(root)
        e.grid(row=row, column=1, columnspan = 2, **widget_global_options)       
        self.widget_infos.append((e, key, validator, descrition_label))
        return e

    def ask_true_false_row(self, text, key, validator, descrition_label):
        root = self.root
        row = self.next_row()
        v = T.BooleanVar()
        v.set(False)
        T.Label(root, text=text).grid(row=row, column=0, **widget_global_options)
        T.Checkbutton(root, text="Enable", \
                          variable=v).grid(row=row, column=1, columnspan=2, **widget_global_options)
        self.widget_infos.append((v, key, validator, descrition_label))
        return v

    def alternatives_row(self, text, lst, key, validator, descrition_label):
        root = self.root
        row = self.next_row()
        class wrap(object):
            def __init__(self, lbox):
                self.lbox = lbox
            def get(self):
                lbox = self.lbox
                return lbox.get(lbox.curselection()[0])
        T.Label(root, text=text).grid(row=row, column = 0, **widget_global_options)
        lbox = T.Listbox(root, selectmode=T.BROWSE, exportselection=0, height=len(lst)+1)
        lbox.grid(row=row, column=1, columnspan = 2, **widget_global_options )
        for i in lst:
            lbox.insert(T.END, i)
            lbox.selection_set(0)
        w = wrap(lbox)
        self.widget_infos.append((w, key, validator, descrition_label))
        return w
