#!/usr/bin/python

try:
    import psyco
    psyco.full()
except:
    print "psyco disabled"

import sys
from optparse import OptionParser
from generalized_multi import glue_rules as app
import os

usage = \
"""
%prog [options] itemsets minconf

Encode itemsets and extract association rules
"""

available_sort_by_strategies = ['lift', 'confidence', 'support']

def shell_ui(parser, options, args):
    if len(args) < 2:
        parser.error("Missing argument")
        sys.exit()

    if options.sort_by.lower() not in available_sort_by_strategies:
        parser.error('sort by can be %s' % ('|'.join(available_sort_by_strategies), ) )

    try:
        dataset = args[0]
        minconf = float(args[1])
        if minconf > 1.0 or minconf < 0:
            raise ValueError
    except ValueError:
        parser.error("minconf must be an integer")

    log_file = options.log_file
    output_file = options.output

    app.main(dataset, minconf, log_file, output_file, sort_by = options.sort_by, 
             ascending = options.ascending,
             opportunistic = options.opportunistic)
    


if __name__ == '__main__':
    # Process arguments
    parser = OptionParser(usage=usage)

    parser.add_option("-g", "--gui", dest="gui",default=False,action="store_true",
              help="Run a minimal GUI.")

    parser.add_option("-l", "--log", dest="log_file",default=os.devnull,
                      help="log on LOGFILE. Default: %s" % (os.devnull,) , metavar="LOGFILE")

    parser.add_option("-s", "--sort-by", dest="sort_by",default="lift",
                      help="Sort by support, confidence, lift (i.e. -s support). Default: lift" , 
                      metavar="(list|confidence|support)")

    parser.add_option("-a", "--ascending", dest="ascending",default=False, action="store_true",
                      help="sort rules in ascending order. Default: False")    

    parser.add_option("-p", "--opportunistic", dest="opportunistic",default=False, action="store_true",
                      help="Dataset contains itemsets in the form X \ S. Default: False")    

    parser.add_option("-o", "--output", dest="output",default=None,
                      help="write itemset to OUTFILE. Default: standard output" , metavar="OUTFILE")

    (options, args) = parser.parse_args()

    if options.gui:
        from generalized_multi import tkgui
        tkgui.ar_gui(parser, options, args, available_sort_by_strategies)
    else:
        shell_ui(parser, options, args)
